#include <stdio.h>
#include <stdlib.h>

void printArray(int size, double data[size])
{
  int i;
  for (i=0;i<size;i++) 
    printf("%.2lf ", data[i]);

  printf("\n");
}
  
void diffArray(int size, double data[size])
{
   int j;

 // #1: memory wasteful solution
 /*
   double tmp[size];

   for (j=0; j<size; j++)
     tmp[j] = data[j];

   for (j=0; j<size; j++)
     data[j] -= tmp[(j+1)%size];
 */

 // #2: another memory wasteful solution
 /*
   double tmp[size];

   for (j=0; j<size-1; j++)
     tmp[j] = data[j+1];
   tmp[size-1] = data[0];

   for (j=0; j<size; j++)
     data[j] -= tmp[j];
 */

 // #3: a cheaper solution, going backward
 /*
   double tmp = data[0];

   for (j=size-1; j>0; j--) {
     double t = data[j];
     data[j] -= tmp;
     tmp = t;
   }

 */

 // #4: the simplest solution
 /*
   double tmp = data[0];

   for (j=; j<size-1; j++)
     data[j] -= data[j+1];
   
   data[size-1] -= tmp;
 */
}

#define NITER (2)
const int size=20;

int main() {
  int i;
  double data[size];

  // initialize array
  for ( i=0; i<size; i++) 
    data[i] = i+1;

  printf("\nInitial values:\n");
  printArray(size, data);

  for (i=1; i<=NITER; i++) {
    diffArray(size, data);
    printArray(size, data);
  }

  return 0;
}   
