#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define EPSILON 1.e-6

double bisection(
  double a, double b, // search interval [a,b]
  double (*f)(double x) // pointer to a function f
) 
{
  double c, delta;

  if ( f(a) * f(b) > 0) {
    fprintf(stderr, "\nSelected interval [a,b] does change sign to function\n");
    return 0.0;
  }

  if (a > b) {
    c = a;
    a = b;
    b = c;
  }

  do {
    double p;
    c = 0.5 * (a + b);
    p = f(a) * f(c);
    if (p > 0.0) {
      a = c;
    } else if (p < 0.0) {
      b = c;
    } else {
      return c; // c is our root
    }

    delta = fabs(a - b);

  } while (delta > EPSILON);

  return c;
}

// a "tricky" function
double f (double x) {
  return exp(-0.1*(x*x))*cos(M_PI*x);
}

int main () {

  double a, b;

  printf("\nFind root with Bisection method of\n");
  printf("f(x)=0 in (a,b)\n");

  printf("\nPlease enter a, b: ");
  scanf("%lf, %lf", &a, &b);

  // using sin as input function to bisection
  printf("\nRoot found at %lf\n", bisection(a, b, sin) );

  // using a user defined function as new input
  printf("\nRoot found at %lf\n", bisection(a, b, f) );
  return 0;
}
