#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define MAXWORDLEN 32
struct tnode {
  char *word;
  int count;
  struct tnode *left;
  struct tnode *right;
};

struct tnode *addtree(struct tnode *, char *);
void treeprint(struct tnode *);
int getword(char *, int);

int main() {

  struct tnode *root;
  char word[MAXWORDLEN];

  root = NULL;
  while (getword(word, MAXWORDLEN) != EOF)
    if (isalpha(word[0]))
      root = addtree(root, word);

  treeprint(root);
  return 0;
}

int getword(char *word, int lim)
{
  int c;
  void ungetch(int);
  char *w = word;

  while (isspace(c = getc(stdin)));

  if (c != EOF)
    *w++ = c;
  if (!isalpha(c)) {
    *w = '\0';
    return c;
  }

  for (; --lim > 0; w++)
    if (!isalnum(*w = getc(stdin))) {
      ungetc(*w, stdin);
      break;
    }
  *w = '\0';

  return word[0];
}

struct tnode *talloc(void);

struct tnode *addtree(struct tnode *p, char *w)
{
  int cond;

  if (p == NULL) {  /* a new word is found */
    p = talloc();   /* make a new node */
    p->word = strdup(w);
    p->count = 1;
    p->left = p->right = NULL;
  } else if ( (cond = strcmp(w, p->word)) == 0)
    p->count++;       /* repeated word */
  else if (cond < 0)  /* look in left subtree */
    p->left = addtree(p->left, w);
  else if (cond > 0   /* look in right subtree */)
    p->right = addtree(p->right, w);

  return p;
}

/* talloc: make a tnode */
struct tnode *talloc (void) {
  return (struct tnode *) malloc (sizeof(struct tnode));
}

/* treeprint: in-order print of tree p */
void treeprint (struct tnode *p)
{
  if (p != NULL) {
    treeprint(p->left);
    printf("%4d %s\n", p->count, p->word);
    treeprint(p->right);
  }
}
