/* fileread: reads an array of double from a text file. 
 * Input:
 *   name: name of the input file;
 *   buffer: the array that stores data;
 *   sz: number of elements to be read;
 * Output:
 *   The number of elements read by the function;
 *   fileread reads as many elements as the minimum 
 *   between sz and input file length.
 *  
 * filewrite: writes an array of double to a text file.
 * Input:
 *   name: name of the output file;
 *   buffer: the array that stores data;
 *   sz: the number of elements to write;
 * Output:
 *   The number of elements written to the file;
 *
 * The text file has the following format:
 * index element
 * fileread ignores index.
*/

#include <stdio.h>
#include <stdlib.h>

int fileread(const char *name, double buffer[], int sz) {
	FILE *fp;
	double *item = buffer;

	if( (fp = fopen(name, "r")) == NULL) {
		perror(name);
		exit(1);
	}

	while( fscanf(fp, "%*d %8lf\n",item) != EOF && item != buffer+sz)
		item++;	

    fclose(fp);

	return (int)(item-buffer);
}

int filewrite(const char *name, double buffer[], int sz) {
	FILE *fp;
	double *item = buffer;

	if( (fp = fopen(name, "w")) == NULL) {
		perror(name);
		exit(1);
	}

	int i;
	for( i=0; i<sz; i++)
		fprintf(fp, "%4d %8lf\n", i, buffer[i]);

    fclose(fp);

	return i;
}
