#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <stdlib.h>

#define BUFFSIZE (256)

int nx, ny;
double tol;

void parseinput(const char *filename)
{
  FILE *parfile;
  char line[BUFFSIZE];
  int nline = 0;

  if ((parfile = fopen(filename, "r")) == NULL) {
    perror(filename);
    exit(1);
  }

  while (fgets(line, sizeof(line), parfile)) {
    char key[BUFFSIZE];
    double val;
    int ret;

    ret = sscanf(line, "%s %lf", key, &val);
    nline++;

    if (*line == '#' || *line == '\n')
      continue;			// skip comment or empty lines

    if (ret != 2) {
      fprintf(stderr, "Invalid format at line %d:\n%s\n", nline, line);
      exit(1);
    }

    if (!strcmp(key, "nx")) {
      if (val < 0) {
	fprintf(stderr,
		"Parameter nx at line %d must be positive integer\n", nline);
	exit(1);
      }
      if (nx != 0) {
	fprintf(stderr,
		"Parameter nx at line %d has been already defined\n", nline);
	exit(1);
      }
      nx = (int) val;
    } else if (!strcmp(key, "ny")) {
      if (val < 0) {
	fprintf(stderr,
		"Parameter ny at line %d must be positive integer\n", nline);
	exit(1);
      }
      if (ny != 0) {
	fprintf(stderr,
		"Parameter ny at line %d has been already defined\n", nline);
	exit(1);
      }
      ny = (int) val;
    } else if (!strcmp(key, "tol")) {
      if (val < 0.0) {
	fprintf(stderr, "Parameter tol at line %d must be positive\n", nline);
	exit(1);
      }
      if (tol != 0.0) {
	fprintf(stderr,
		"Parameter tol at line %d has been already defined\n", nline);
	exit(1);
      }
      tol = val;
    } else {
      fprintf(stderr, "Unknown parameter name\n");
      exit(1);
    }
  }
}

int main(int argc, char *argv[])
{

  if (argc < 2) {
    fprintf(stderr, "Missing input file name\n");
    exit(1);
  }

  if (argc > 2) {
    fprintf(stderr, "Too many parameters\n");
    exit(1);
  }

  parseinput(argv[1]);
  printf("%d %d %lf\n", nx, ny, tol);

  return 0;
}
