#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#define BUFFSIZE (256)

int nx, ny;
double tol;

int parseposint(const char *s, const char *param, int n, int nline)
{
  long val;
  char *p = (char *) s;

  if (n != 0) {
    fprintf(stderr, "Line %d: parameter %s already defined\n", nline, param);
    exit(1);
  }

  errno = 0;
  val = strtol(s, &p, 10);
  if (errno == ERANGE) {
    fprintf(stderr, "Line %d: value is out of range\n", nline);
    exit(1);
  }

  if (p == s || *p != 0 && *p != '\n') {
    fprintf(stderr, "Line %d: invalid format\n", nline);
    exit(1);
  }

  if (val < 0) {
    fprintf(stderr, "Line %d: parameter %s must be positive\n", nline, param);
    exit(1);
  }

  return val;
}

double parseposdouble(const char *s, const char *param, double n, int nline)
{
  double val;
  char *p = (char *) s;

  if (n != 0.0) {
    fprintf(stderr, "Line %d: parameter %s already defined\n", nline, param);
    exit(1);
  }

  errno = 0;
  val = strtod(s, &p);
  if (errno == ERANGE) {
    fprintf(stderr, "Line %d: value is out of range\n", nline);
    exit(1);
  }

  if (p == s || *p != 0 && *p != '\n') {
    fprintf(stderr, "Line %d: invalid format\n", nline);
    exit(1);
  }

  if (val < 0.0) {
    fprintf(stderr, "Line %d: parameter %s must be positive\n", nline, param);
    exit(1);
  }

  return val;
}

void parseinput(const char *filename)
{
  FILE *parfile;
  char line[BUFFSIZE];
  int nline = 0;

  if ((parfile = fopen(filename, "r")) == NULL) {
    perror(filename);
    exit(1);
  }

  while (fgets(line, sizeof(line), parfile)) {
    double val;

    nline++;

    if (*line == '#' || *line == '\n')
      continue;			// comment line

    if (!strncmp(line, "nx", 2))
      nx = parseposint(line + 2, "nx", nx, nline);
    else if (!strncmp(line, "ny", 2))
      ny = parseposint(line + 2, "ny", ny, nline);
    else if (!strncmp(line, "tol", 3))
      tol = parseposdouble(line + 3, "tol", tol, nline);
    else {
      fprintf(stderr, "Line %d: unknown parameter name\n", nline);
      exit(1);
    }
  }
}

int main(int argc, char *argv[])
{

  if (argc < 2) {
    fprintf(stderr, "Missing input file name\n");
    exit(1);
  }

  if (argc > 2) {
    fprintf(stderr, "Too many parameters\n");
    exit(1);
  }

  parseinput(argv[1]);
  printf("%d %d %lf\n", nx, ny, tol);

  return 0;
}
