#include <stdio.h>
#include <stdlib.h>

void printArray(int size, double data[size])
{
   int i;
   for (i=0;i<size;i++) 
      printf("%.2lf ", data[i]);
   printf("\n");
}
  
void smoothing(int size, int ns, double data[size])
{
  int j, k;
  double norm = 1.0/(2.0*ns+1);
  double *wrk;

  wrk = (double *) malloc (size * sizeof(double));
  if (wrk == NULL) {
    fprintf(stderr, "\nMemory Allocation Failure in smoothing function\n");
    exit(EXIT_FAILURE);
  }

  for (j=0; j<size; j++) {
     wrk[j] = 0.0;
     for (k=j-ns; k<=j+ns; k++) 
        wrk[j] = wrk[j] + data[(size+k)%size];
  }

  for (j=0; j<size; j++) {
     data[j] = norm * wrk[j];
  }   

  // remember to free what you dynamically allocate !
  free(wrk);
}

#define NITER (2)

int main() {
   int i, size, ns;
   double *data;

   printf("\nenter the array size: ");
   scanf("%d", &size);

   data = (double *) malloc (size * sizeof(double));
   if (data == NULL) {
     fprintf(stderr, "\nMemory Allocation Failure in main\n");
     exit(EXIT_FAILURE);
   }

   // initialize array
   for ( i=0;i<size;i++) 
      data[i] = i+1;

   printf("enter the number of smoothing points, array size is %d:\n", size);
   scanf("%d", &ns);

   if ( ns > size) {
      fprintf(stderr, "Error: More smoothing points than array elements.\n");
      exit(1);
   }

   printf("\nInitial values:\n");
   printArray(size, data);

   for (i=1;i<=NITER;i++) {
    smoothing(size, ns, data);
    printArray(size, data);
   }

   // data would be deallocated anyway on program exit
   free(data);

   return 0;
}   
