#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>

struct vec3d {
  double x, y, z;
};

typedef struct vec3d vec3d;

double dot_product(const vec3d *a, const vec3d *b) {
  double dot;
  dot = (a->x)*(b->x) + 
        (a->y)*(b->y) + 
        (a->z)*(b->z);

  return dot;
}

double vec3d_modulus(const vec3d *a) {
  return sqrt(dot_product(a, a));
}

int modulus_compare(const vec3d *a, const vec3d *b) {
  double mod_a, mod_b; 
  mod_a = dot_product(a, a);
  mod_b = dot_product(b, b);

  if (mod_a == mod_b)
    return 0;

  if (mod_a > mod_b)
    return 1;

  return -1;
}

void print_vec3d(vec3d *a) {
  printf("(%lf,%lf,%lf)[%lf]", 
    a->x, a->y, a->z, vec3d_modulus(a));
}

void print_array_vec3d(unsigned long size, vec3d a[size])
{
  unsigned long i;
  for (i=0; i<size; i++) {
    print_vec3d(a+i); // passing address of i-th element
    printf("\n");
  }
  printf("\n");
}


// Example main program using vec3d

#define SIZE 4 

int main() {
  unsigned long i;
  const double rand_norm = 10.0/(RAND_MAX);

  vec3d *a = (vec3d *) malloc (SIZE*sizeof(vec3d));
  if (a == NULL) {
    printf("Memory allocation error. Sorry ...\n");
    exit(EXIT_FAILURE);
  }

  if (ULONG_MAX < SIZE) {
    printf("Array size too big to be represented\n");
    exit(EXIT_FAILURE);
  }

  // initialize vec3d componets in [0,10]
  for (i=0; i<SIZE; i++) {
    a[i].x = rand_norm*rand();
    a[i].y = rand_norm*rand();
    a[i].z = rand_norm*rand();
  }

  // print starting vec3d array elements
  print_array_vec3d(SIZE, a);

  printf("Sorting vec3d array ... ");
  qsort(a, SIZE, sizeof(vec3d), modulus_compare);
  printf("done!\n\n");

  // print sorted vec3d array elements
  print_array_vec3d(SIZE, a);

  // free our dynamically allocated memory
  free(a);
  // anyway here is useless since 
  // we are exiting the program

  return 0;
}
